@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 from __future__ import print_function
 
 import doctest
@@ -99,6 +97,14 @@ def doctest_LogParser():
         >>> test('[15 Jan 08:42] <jsmith!n=jsmith@10.20.30.40> Hello!')
         '15 Jan 08:42' COMMENT ('jsmith', 'Hello!')
 
+    Greater-than signs do not confuse the parser (any more)
+
+        >>> test('14:18 <mg> apples > oranges')
+        '14:18' COMMENT ('mg', 'apples > oranges')
+
+        >>> test('[15 Jan 08:42] <jsmith!n=jsmith@10.20.30.40> Hello > world!')
+        '15 Jan 08:42' COMMENT ('jsmith', 'Hello > world!')
+
     `what` can be COMMENT...
 
         >>> test('<nick> text')
@@ -191,6 +197,11 @@ def doctest_LogParser_encodings():
         >>> test(b'14:18 <mg> cp1252: \x9a')
         '14:18' COMMENT ('mg', 'cp1252: \u0161')
 
+    For convenience, Unicode is also accepted
+
+        >>> test(u'14:18 <mg> hello, \u2603')
+        '14:18' COMMENT ('mg', 'hello, \u2603')
+
     """
 
 
@@ -489,6 +500,9 @@ def doctest_XHTMLStyle():
         >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
         <p id="t02:25" class="action"><a href="#t02:25" class="time">02:25</a> * mgedmin hacks &lt;&amp;&gt;</p>
 
+        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin is very fast')
+        <p id="t02:25-2" class="action"><a href="#t02:25-2" class="time">02:25</a> * mgedmin is very fast</p>
+
         >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
         <p id="t02:26:01" class="part"><a href="#t02:26:01" class="time">02:26</a> * mgedmin leaves</p>
 
@@ -612,7 +626,7 @@ def doctest_MediaWikiStyle():
         |- id="t02:24:17"
         ! style="background-color: #77ff77" | mgedmin
         | style="color: #77ff77" | Hello, world!
-        || [[#t02:24:17|02:24]] 
+        || [[#t02:24:17|02:24]]
 
     Note that we don't need special markup for hyperlinks
 
@@ -620,7 +634,7 @@ def doctest_MediaWikiStyle():
         |- id="t02:24"
         ! style="background-color: #77ff77" | mgedmin
         | style="color: #77ff77" | http://google.com/ has a new favicon
-        || [[#t02:24|02:24]] 
+        || [[#t02:24|02:24]]
 
     But we ought to escape MediaWiki markup (XXX this is not currently done,
     would someone familiar with the markup please fix the code)
@@ -629,14 +643,14 @@ def doctest_MediaWikiStyle():
         |- id="t02:24"
         ! style="background-color: #77ff77" | [|mg|]
         | style="color: #77ff77" | [!@#$%^&amp;*()_+{};:,./&lt;&gt;?]
-        || [[#t02:24|02:24]] 
+        || [[#t02:24|02:24]]
 
     The time is optional (some IRC logs don't have it)
 
         >>> style.nicktext(None, 'mgedmin', 'what time is it?', '#77ff77')
         |-
         | style="background-color: #77ff77" | mgedmin
-        | style="color: #77ff77" colspan="2" | what time is it? 
+        | style="color: #77ff77" colspan="2" | what time is it?
 
     There are other kinds of things that happen in IRC channels
 
@@ -857,9 +871,6 @@ def doctest_main_output_directory():
 def test_suite():
     optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS | doctest.REPORT_NDIFF
     return unittest.TestSuite([
-                doctest.DocTestSuite('irclog2html.irclog2html'),
-                doctest.DocTestSuite(optionflags=optionflags)])
-
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='test_suite')
+        doctest.DocTestSuite('irclog2html.irclog2html'),
+        doctest.DocTestSuite(optionflags=optionflags),
+    ])
