@@ -1,17 +1,21 @@
+import cgi
 import datetime
 import doctest
 import gzip
-import io
 import os
+import re
 import shutil
 import sys
 import tempfile
 import unittest
 from contextlib import closing
 
+import mock
+from zope.testing import renormalizing
+
 from irclog2html.irclogsearch import (
     SearchResult, SearchResultFormatter, LogParser, search_irc_logs,
-    print_search_form, print_search_results, main)
+    print_search_form, print_search_results, search_page, main)
 
 
 try:
@@ -48,6 +52,14 @@ class BytesIOWrapper(object):
         self.stream.flush()
 
 
+def prepare_stdout():
+    if sys.version_info[0] >= 3:
+        # Make the StringIO used by doctest act like the real sys.stdout, which
+        # has an underlying bytes-only output stream.  This is needed because
+        # we're going to print bytes in this CGI script.
+        sys.stdout.buffer = BytesIOWrapper(sys.stdout)
+
+
 def myrepr(o):
     """Repr that drops u prefixes on unicode strings."""
     if isinstance(o, tuple):
@@ -64,7 +76,8 @@ def myrepr(o):
 def doctest_SearchResultFormatter():
     """Test for SearchResultFormatter
 
-        >>> srf = SearchResultFormatter(BytesIOWrapper(sys.stdout))
+        >>> sys.stdout.buffer = BytesIOWrapper(sys.stdout)
+        >>> srf = SearchResultFormatter(sys.stdout)
         >>> srf.print_prefix()
         <table class="irclog">
 
@@ -76,15 +89,15 @@ def doctest_SearchResultFormatter():
 
         >>> srf.print_html(SearchResult(
         ...     filename='/path/to/log', link='log.html',
-        ...     date=datetime.date(2013, 3, 17), time='12:34',
+        ...     date=datetime.date(2013, 3, 17), time='12:35',
         ...     event=LogParser.NICKCHANGE, info=('mgedmin is now known as mg_away', 'mgedmin', 'mg_away')))
-        <tr id="t12:34"><td class="nickchange" colspan="2">mgedmin is now known as mg_away</td><td><a href="log.html#t12:34" class="time">12:34</a></td></tr>
+        <tr id="t12:35"><td class="nickchange" colspan="2">mgedmin is now known as mg_away</td><td><a href="log.html#t12:35" class="time">12:35</a></td></tr>
 
         >>> srf.print_html(SearchResult(
         ...     filename='/path/to/log', link='log.html',
-        ...     date=datetime.date(2013, 3, 17), time='12:34',
+        ...     date=datetime.date(2013, 3, 17), time='12:36',
         ...     event=LogParser.ACTION, info='* mgedmin jumps up and down'))
-        <tr id="t12:34"><td class="action" colspan="2">* mgedmin jumps up and down</td><td><a href="log.html#t12:34" class="time">12:34</a></td></tr>
+        <tr id="t12:36"><td class="action" colspan="2">* mgedmin jumps up and down</td><td><a href="log.html#t12:36" class="time">12:36</a></td></tr>
 
         >>> srf.print_suffix()
         </table>
@@ -129,12 +142,24 @@ def doctest_search_irc_logs():
     """
 
 
+def doctest_search_irc_logs_limit():
+    """Test for search_irc_logs
+
+        >>> tmpdir = set_up_sample()
+        >>> for r in search_irc_logs('seen', where=tmpdir, limit=2):
+        ...     print('%s %s %s %s %s' % (r.link, r.date, r.time, r.event, myrepr(r.info)))
+        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:17 COMMENT ('mgedmin', 'seen mgedmin')
+        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:19 COMMENT ('mgedmin', '!seen mgedmin')
+
+        >>> clean_up_sample(tmpdir)
+
+    """
+
+
 def doctest_print_search_form():
     """Test for print_search_form
 
         >>> print_search_form()
-        Content-Type: text/html; charset=UTF-8
-        <BLANKLINE>
         <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
         <html>
@@ -167,9 +192,7 @@ def doctest_print_search_results():
 
         >>> tmpdir = set_up_sample()
         >>> sys.stdout.buffer = BytesIOWrapper(sys.stdout)
-        >>> print_search_results('povbot', where=tmpdir)
-        Content-Type: text/html; charset=UTF-8
-        <BLANKLINE>
+        >>> fmt = print_search_results('povbot', where=tmpdir)
         <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
         <html>
@@ -197,10 +220,10 @@ def doctest_print_search_results():
           </li>
           <li><a href="sample-2013-03-17.log.html">2013-03-17 (Sunday)</a>:
         <table class="irclog">
-        <tr id="t2005-01-08T23:33:54"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54" class="time">23:33</a></td></tr>
-        <tr id="t2005-01-08T23:46:35"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35" class="time">23:46</a></td></tr>
-        <tr id="t2005-01-08T23:47:19"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19" class="time">23:47</a></td></tr>
-        <tr id="t2005-01-08T23:47:54"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54" class="time">23:47</a></td></tr>
+        <tr id="t2005-01-08T23:33:54-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54-2" class="time">23:33</a></td></tr>
+        <tr id="t2005-01-08T23:46:35-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35-2" class="time">23:46</a></td></tr>
+        <tr id="t2005-01-08T23:47:19-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19-2" class="time">23:47</a></td></tr>
+        <tr id="t2005-01-08T23:47:54-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54-2" class="time">23:47</a></td></tr>
         </table>
           </li>
         </ul>
@@ -218,11 +241,44 @@ def doctest_print_search_results():
     """
 
 
+def doctest_search_page():
+    """Test search_page
+    Let's mock the dependency functions:
+
+        >>> patcher = mock.patch.multiple("irclog2html.irclogsearch",
+        ...                               print_search_form=mock.DEFAULT,
+        ...                               print_search_results=mock.DEFAULT)
+
+        >>> values = patcher.start()
+
+    When there is a 'q' param in the GET query, the logs are searched:
+
+        >>> form = cgi.FieldStorage(
+        ...     environ={'QUERY_STRING': 'q=123', 'HTTP_METHOD': 'GET'})
+        >>> search_page("The stream", form, "/logs", "#dev*.logs")
+        >>> values['print_search_results'].assert_called_once_with(
+        ...     '123', logfile_pattern='#dev*.logs',
+        ...     stream='The stream', where='/logs')
+
+    When there is no query, the search form is displayed:
+
+        >>> form = cgi.FieldStorage(environ={
+        ...     'HTTP_METHOD': 'GET', 'QUERY_STRING': ''})
+        >>> search_page("The stream", form, "/logs", "#dev*.logs")
+        >>> values['print_search_form'].assert_called_once_with('The stream')
+
+    Clean up:
+
+        >>> patcher.stop()
+
+    """
+
+
 def doctest_main_prints_form():
     """Test for main
 
         >>> os.environ.pop('QUERY_STRING', None)
-        >>> sys.stdout = io.TextIOWrapper(BytesIOWrapper(sys.stdout)) # it's gonna be rewrapped
+        >>> prepare_stdout()
         >>> main()
         Content-Type: text/html; charset=UTF-8
         <BLANKLINE>
@@ -257,7 +313,7 @@ def doctest_main_searches():
     """Test for main
 
         >>> tmpdir = set_up_sample()
-        >>> sys.stdout = io.TextIOWrapper(BytesIOWrapper(sys.stdout)) # it's gonna be rewrapped
+        >>> prepare_stdout()
         >>> os.environ['QUERY_STRING'] = 'q=povbot'
         >>> os.environ['IRCLOG_LOCATION'] = tmpdir
         >>> os.environ['IRCLOG_GLOB'] = '*.log'
@@ -291,10 +347,10 @@ def doctest_main_searches():
           </li>
           <li><a href="sample-2013-03-17.log.html">2013-03-17 (Sunday)</a>:
         <table class="irclog">
-        <tr id="t2005-01-08T23:33:54"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54" class="time">23:33</a></td></tr>
-        <tr id="t2005-01-08T23:46:35"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35" class="time">23:46</a></td></tr>
-        <tr id="t2005-01-08T23:47:19"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19" class="time">23:47</a></td></tr>
-        <tr id="t2005-01-08T23:47:54"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54" class="time">23:47</a></td></tr>
+        <tr id="t2005-01-08T23:33:54-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54-2" class="time">23:33</a></td></tr>
+        <tr id="t2005-01-08T23:46:35-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35-2" class="time">23:46</a></td></tr>
+        <tr id="t2005-01-08T23:47:19-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19-2" class="time">23:47</a></td></tr>
+        <tr id="t2005-01-08T23:47:54-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54-2" class="time">23:47</a></td></tr>
         </table>
           </li>
         </ul>
@@ -311,11 +367,19 @@ def doctest_main_searches():
     """
 
 
+checker = None
+if sys.version_info[0] == 2:
+    checker = renormalizing.RENormalizing([
+        (re.compile(r"^\['"), r"[b'"),
+        (re.compile(r"u('.*?')"), r"\1"),
+    ])
+
+
 def test_suite():
     optionflags = (doctest.ELLIPSIS | doctest.REPORT_NDIFF |
                    doctest.NORMALIZE_WHITESPACE)
     return unittest.TestSuite([
-        doctest.DocTestSuite(optionflags=optionflags),
+        doctest.DocTestSuite(optionflags=optionflags, checker=checker),
     ])
 
 
